DECLARE @MinDateTime DATETIME = CAST(-53690 AS DATETIME);
DECLARE @SysWorkspaceDefault NVARCHAR(250) = 'Default';
DECLARE @InstallTypeSourceControl INT = 0;
DECLARE @InstallTypeRepository INT = 1;

WITH BasePackage(Id, Name, InstallType, IsLocked, IsChanged, Version, Maintainer) AS (
	SELECT
		sp.Id, sp.Name, InstallType, IsLocked, IsChanged, sp.Version, Maintainer
	FROM SysPackage sp WITH(NOLOCK)
	JOIN SysWorkspace sw ON sp.SysWorkspaceId = sw.Id
	WHERE sw.Name = @SysWorkspaceDefault
		AND sp.Name <> 'Custom'
		AND sp.Maintainer = 'Terrasoft'
),
UnlockCustomerPackagesSchemasCount(Id, Name, Count) AS (
	SELECT
		BasePackage.Id, BasePackage.Name, COUNT(BasePackage.Id)
	FROM SysSchema ss WITH(NOLOCK)
	INNER JOIN BasePackage ON BasePackage.Id = ss.SysPackageId
	WHERE InstallType = @InstallTypeSourceControl
		AND (ss.IsLocked = 1 AND ss.IsChanged = 1)
	GROUP BY BasePackage.Id, BasePackage.Name
),
AllCustomerPackagesSchemasCount(Id, Name, Count) AS (
	SELECT
		BasePackage.Id, BasePackage.Name, COUNT(BasePackage.Id)
	FROM SysSchema ss WITH(NOLOCK)
	INNER JOIN BasePackage ON BasePackage.Id = ss.SysPackageId
	WHERE InstallType = @InstallTypeSourceControl
	GROUP BY BasePackage.Id, BasePackage.Name
),
CustomerPackages(Id, Name) AS (
	SELECT
		BasePackage.Id, BasePackage.Name
	FROM SysSchema ss WITH(NOLOCK)
	INNER JOIN BasePackage ON BasePackage.Id = ss.SysPackageId
	INNER JOIN AllCustomerPackagesSchemasCount ON BasePackage.Id = AllCustomerPackagesSchemasCount.Id
	INNER JOIN UnlockCustomerPackagesSchemasCount ON BasePackage.Id = UnlockCustomerPackagesSchemasCount.Id
	WHERE InstallType = @InstallTypeSourceControl
		AND UnlockCustomerPackagesSchemasCount.Count > 0
		AND UnlockCustomerPackagesSchemasCount.Count = AllCustomerPackagesSchemasCount.Count
	GROUP BY BasePackage.Id, BasePackage.Name
),
UnlockPackage(Id, Name, IsLocked, IsChanged, Version, Maintainer, InstallType) AS (
	SELECT
		BasePackage.Id, BasePackage.Name, BasePackage.IsLocked, BasePackage.IsChanged, Version, Maintainer, InstallType
	FROM BasePackage
	LEFT JOIN CustomerPackages ON CustomerPackages.Id = BasePackage.Id
	WHERE InstallType = @InstallTypeSourceControl
		AND CustomerPackages.Id IS NULL
),
UnlockSchema(Id, Name, PackageName, IsLocked, IsChanged) AS (
	SELECT
		ss.Id, ss.Name, BasePackage.Name, ss.IsLocked, ss.IsChanged
	FROM SysSchema ss WITH(NOLOCK)
	INNER JOIN BasePackage ON BasePackage.Id = ss.SysPackageId
	LEFT JOIN CustomerPackages ON CustomerPackages.Id = BasePackage.Id
	WHERE ss.IsLocked = 1 AND ss.IsChanged = 1
		--AND ss.ManagerName <> 'ProcessSchemaManager'
		AND CustomerPackages.Id IS NULL
)
--      
SELECT
	DB_NAME() AS DBName,
	Name AS PackageName,
	NULL AS SchemaName,
	Version AS VersionPack,
	Maintainer AS Maintainer,
	IsLocked,
	IsChanged,
	InstallType AS InstallType,
	Id
INTO #UnlockSchemaAndPackages
FROM UnlockPackage
UNION ALL
SELECT 
	DB_NAME() AS DBName,
	PackageName AS PackageName,
	Name AS SchemaName,
	NULL AS VersionPack,
	NULL AS Maintainer,
	IsLocked,
	IsChanged,
	NULL AS InstallType,
	Id
FROM UnlockSchema;

--region      
UPDATE SysSchema
SET
	ModifiedOn = @MinDateTime,
	MetaDataModifiedOn = @MinDateTime
FROM SysSchema ss
JOIN SysPackage sp ON ss.SysPackageId = sp.Id
WHERE sp.Name = 'NUI'
	AND ss.Name = 'ConfigurationSectionHelper';

UPDATE SysPackage
SET
	ModifiedOn = @MinDateTime
WHERE Name = 'NUI';
--endregion

--region 
-- 
UPDATE SysPackage
SET
	IsChanged = 0, IsLocked = 0, InstallType = @InstallTypeRepository
FROM SysPackage sp
JOIN #UnlockSchemaAndPackages unlock ON sp.Id = unlock.Id
WHERE SchemaName IS NULL;

-- 
UPDATE SysSchema
SET
	IsChanged = 0, IsLocked = 0
FROM SysSchema ss
JOIN #UnlockSchemaAndPackages unlock ON ss.Id = unlock.Id
WHERE unlock.InstallType IS NULL;
--endregion

DROP TABLE #UnlockSchemaAndPackages;